@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.887';
+our $VERSION = '1.896';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -55,7 +55,7 @@ sub new {
    my $impl = defined( $argv{impl} ) ? ucfirst( lc $argv{impl} ) : 'Mutex';
 
    # Replace 'fast' with 'Fast' in the implementation value.
-   $impl =~ s/fast$/Fast/;
+   $impl =~ s/fast/Fast/;
 
    $impl = 'Threads'     if ( $impl eq 'Mutex' && $^O eq 'MSWin32' );
    $impl = 'ThreadsFast' if ( $impl eq 'MutexFast' && $^O eq 'MSWin32' );
@@ -131,7 +131,7 @@ MCE::Channel - Queue-like and two-way communication capability
 
 =head1 VERSION
 
-This document describes MCE::Channel version 1.887
+This document describes MCE::Channel version 1.896
 
 =head1 SYNOPSIS
 
@@ -735,7 +735,7 @@ Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019-2023 by Mario E. Roy
+Copyright (C) 2019-2024 by Mario E. Roy
 
 MCE::Channel is released under the same license as Perl.
 
