@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.884';
+our $VERSION = '1.896';
 
 use base 'MCE::Channel';
 
@@ -227,16 +227,8 @@ sub send2 {
    my $data = $freeze->([ @_ ]);
 
    local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
-
-   {
-      local $MCE::Signal::IPC = 1;
-
-      MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
-      print { $self->{c_sock} } pack('i', length $data) . $data;
-   }
-
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+   MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
+   print { $self->{c_sock} } pack('i', length $data) . $data;
 
    return 1;
 }
@@ -300,7 +292,7 @@ MCE::Channel::Simple - Channel tuned for one producer and one consumer
 
 =head1 VERSION
 
-This document describes MCE::Channel::Simple version 1.884
+This document describes MCE::Channel::Simple version 1.896
 
 =head1 DESCRIPTION
 
